package bus

import (
	"context"
	"pay/db/model"
	"pay/utils"

	"github.com/nats-io/nats.go"
	"infish.cn/comm"
	"infish.cn/comm/pay"
)

type SinglePay struct{}

// 创建二维码
func (s *SinglePay) CreateQr() *comm.NatsMsgReplyer {
	utils.ConsoleFormat(pay.PaySingleQrApi)

	return &comm.NatsMsgReplyer{
		Subject: pay.PaySingleQrApi,
		Entity:  func() interface{} { return &pay.OrderMsg{} },
		Cb2: func(_ *nats.Msg, entity interface{}) (interface{}, error) {
			m := entity.(*pay.OrderMsg)
			pay := PayMod(int(*m.PayMod))
			ctx := context.Background()
			res, err := pay.Pay(ctx, m)
			return res, err
		},
	}
}

// 关闭订单应答
func (s *SinglePay) CloseOrder() *comm.NatsMsgReplyer {
	utils.ConsoleFormat(pay.PaySingleCloseOrderApi)

	return &comm.NatsMsgReplyer{
		Subject: pay.PaySingleCloseOrderApi,
		Entity:  func() interface{} { return &pay.OrderMsg{} },
		Cb2: func(_ *nats.Msg, entity interface{}) (interface{}, error) {
			m := entity.(*pay.OrderMsg)
			pay := PayMod(int(*m.PayMod))
			ctx := context.Background()
			flag, err := pay.Close(ctx, m)
			return flag, err
		},
	}
}

// 退款
func (s *SinglePay) Refund() *comm.NatsMsgReplyer {
	utils.ConsoleFormat(pay.PaySingleRefundApi)
	refund := pay.ReFund{}
	return &comm.NatsMsgReplyer{
		Subject: pay.PaySingleRefundApi,
		Entity:  func() interface{} { return &refund },
		Cb2: func(_ *nats.Msg, entity interface{}) (interface{}, error) {
			refund := entity.(*pay.ReFund)
			// ???适配缺失退款理由
			if refund.Reason == "" {
				refund.Reason = "默认退款"
			}
			pay := PayMod(int(*refund.PayMod))
			ctx := context.Background()
			flag, err := pay.ReFundPay(ctx, refund)
			return flag, err
		},
	}
}

// 退款查询
func (s *SinglePay) RefundQuery() *comm.NatsMsgReplyer {
	utils.ConsoleFormat(pay.PaySingleRefundQueryApi)

	return &comm.NatsMsgReplyer{
		Subject: pay.PaySingleRefundQueryApi,
		Entity:  func() interface{} { return &model.ReFund{} },
		Cb2: func(_ *nats.Msg, entity interface{}) (interface{}, error) {
			m := entity.(*model.ReFund)
			pay := PayMod(int(*m.PayMode))
			ctx := context.Background()
			status, err := pay.ReFundQuery(ctx, m)
			return status, err
		},
	}
}
